// Copyright 1998-2019 Epic Games, Inc. All Rights Reserved.

#include "StoveSDKPlugin.h"
#include "Core.h"
#include "Modules/ModuleManager.h"
#include "Interfaces/IPluginManager.h"
#include "GenericPlatform/GenericPlatformMisc.h"
#include "../../../Binaries/Thirdparty/StoveSDKPluginLibrary/include/StovePCSDK.h"


#define LOCTEXT_NAMESPACE "FStoveSDKPluginModule"

void FStoveSDKPluginModule::StartupModule()
{
    // This code will execute after your module is loaded into memory; the exact timing is specified in the .uplugin file per-module

    // Get the base directory of this plugin
    FString baseDir = IPluginManager::Get().FindPlugin("StoveSDKPlugin")->GetBaseDir();
    FString libraryPath;
    FString libraryName = "StovePCSDK.dll";
    // Add on the relative location of the third party dll and load it
#ifdef _WIN64
    libraryPath = FPaths::Combine(*baseDir, TEXT("Binaries/ThirdParty/StoveSDKPluginLibrary/bin/x64/Release/"));
#else
    libraryPath = FPaths::Combine(*baseDir, TEXT("Binaries/ThirdParty/StoveSDKPluginLibrary/bin/Win32/Release/"));
#endif

    FPlatformProcess::PushDllDirectory(*libraryPath);
    _stoveSDKLibraryHandle = !libraryPath.IsEmpty() ? FPlatformProcess::GetDllHandle(*(libraryPath + libraryName)) : nullptr;
}


void FStoveSDKPluginModule::ShutdownModule()
{
    // This function may be called during shutdown to clean up your module.  For modules that support dynamic reloading,
    // we call this function before unloading the module.

    // Free the dll handle
    FPlatformProcess::FreeDllHandle(_stoveSDKLibraryHandle);
    _stoveSDKLibraryHandle = nullptr;
}

StovePCResult FStoveSDKPluginModule::Init(const StovePCConfig& config, const StovePCCallback& callback)
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : StovePC_Init -->"));
    if (_stoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_Init(config, callback);
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_Init is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_Init is result = %d"), result);
        }
        return result; 
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
    
}

StovePCInitState FStoveSDKPluginModule::InitState()
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : InitState -->"));
    if (_stoveSDKLibraryHandle)
    {
        StovePCInitState state = StovePC_GetInitState();
        if (state != STOVE_PC_INIT_COMPLETE)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_GetInitState is error = %d"), state);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_GetInitState is result = %d"), state);
        }
        return state;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_INIT_FAIL;
    }
}

StovePCResult FStoveSDKPluginModule::RunCallback()
{
    static bool logOnce = true;
    if (logOnce)
    {
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : RunCallback -->"));
    }
    if (_stoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_RunCallback();
        if (result != STOVE_PC_NO_ERROR)
        {
            if (logOnce)
            {
                UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_RunCallback is error = %d"), result);
            }
            
        }
        else
        {
            if (logOnce)
            {
                UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_RunCallback is result = %d"), result);
            }
            
        }
        logOnce = false; 
        return result;
    }
    else
    {
        logOnce = false;
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
    
}

StovePCResult FStoveSDKPluginModule::GetToken()
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : GetToken -->"));
    if (_stoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_GetToken();
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_GetToken is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_GetToken is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

StovePCResult FStoveSDKPluginModule::GetUser()
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : GetUser -->"));
    if (_stoveSDKLibraryHandle)
    {
        StovePCResult result =  StovePC_GetUser();
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_GetUser is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_GetUser is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

StovePCResult FStoveSDKPluginModule::GetOwnership()
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : GetOwnership -->"));
    if (_stoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_GetOwnership();
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_GetOwnership is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_GetOwnership is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

StovePCResult FStoveSDKPluginModule::UnInit()
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : UnInit -->"));
    if (_stoveSDKLibraryHandle)
    {
        // Call the test function in the third party library that opens a message box
        StovePCResult result = StovePC_UnInit();
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_UnInit is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_UnInit is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

#undef LOCTEXT_NAMESPACE
	
IMPLEMENT_MODULE(FStoveSDKPluginModule, StoveSDKPlugin)

DEFINE_LOG_CATEGORY(LogStoveSDKPlugin);
